@@ -21,7 +21,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #ifdef HAVE_CONFIG_H
-  #include <cheese-config.h>
+  #include <config.h>
 #endif
 
 #include <string.h>
